/* Kelsey Reiman
   July 2014
   
   Creates an object like the Minecraft character Steve. Takes in a width, a height and
   a depth as parameters and an image for a "skin." The texture mapping is done using uv
   texture coordinates using the technique described in: 
   
   http://solutiondesign.com/webgl-and-three-js-texture-mapping/
   
*/

Steve = function (w , h, d, skinImagePath){
	
	THREE.Object3D.call(this);

	this.w = w;
	this.h = h;
	this.d = d;
	this.skinImagePath = skinImagePath;

	/**the following four variables are like containers to hold a collection
	of objects that make up Steve. legs hold the left leg and the right leg,
	torso holds the arms and the chest, and the upper body holds the head and 
	and torso. Steve holds the upper body and the legs. Organizing Steve into
	a hierarchical collection of objects makes it easier to animate him or duplicate
	him within the scene. */
	
	var steveObject = new THREE.Object3D();
	var upperBody = new THREE.Object3D();
	var torso = new THREE.Object3D();
	var legs = new THREE.Object3D(); 
	
	
	//Set up the proportions for Steves Body Parts
	
	
	/* The height of the head is 2/8 the total height, while the length of the 
	torso (chest + arms) and legs are each 3/8 of the height. So 2/8 + 3/8 + 3/8  = 8/8. You could
	easily change this if you want to change steve's proportions. */
	var limbH = (3.0/8.0) * h;
	var headH = (2.0/8.0) * h; 
	
	/* The width of the limbs is 1/4 the total width while the widths of the head
	and chest are 1/2 the total width */
	var limbW = (1.0/4.0) * w;
	var chestW = (1.0/2.0) * w;
	var headW = (1.0/2.0) * w;
	
	/* the depth of the head is the same as the total depth, while the depth of 
	the limbs and chest is just 1/2 the total depth. */
	
	var limbD = (1.0/2.0) * d;
	var headD = d;
	 

	
	/**** Load the texture image, assign it to a material and create arrays that
	hold the texture coordinates with names that correspond to each body part that
	part of the texture should be mapped to */
	
	//load the skin texture and assign it to a material
	var texture = THREE.ImageUtils.loadTexture(skinImagePath);
	var steveMat = new THREE.MeshBasicMaterial({map: texture }); 
	
	//assign the texture coordinates to the corresponding parts
	
	//head
	var headLeft  = [new THREE.Vector2(0.0,1.0),new THREE.Vector2(0.0,0.75),
					 new THREE.Vector2(0.25,0.75),new THREE.Vector2(0.25,1.0)];
	var headFront = [new THREE.Vector2(0.25,1.0),new THREE.Vector2(0.25,0.75),
					 new THREE.Vector2(0.5,0.75),new THREE.Vector2(0.5,1.0)];
	var headRight = [new THREE.Vector2(0.5,1.0),new THREE.Vector2(0.5,0.75),
					 new THREE.Vector2(0.75,0.75),new THREE.Vector2(0.75,1.0)];
	var headBack  = [new THREE.Vector2(0.75,1.0),new THREE.Vector2(0.75,0.75),
					 new THREE.Vector2(1.0,0.75),new THREE.Vector2(1.0,1.0)];
	var headTop   = [new THREE.Vector2(0.0,0.75),new THREE.Vector2(0.0,0.5),
					 new THREE.Vector2(0.25,0.5),new THREE.Vector2(0.25,0.75)];
	var headBottom = [new THREE.Vector2(0.25,0.75),new THREE.Vector2(0.25,0.5),
					 new THREE.Vector2(0.5,0.5),new THREE.Vector2(0.5,0.75)];
					 
	//torso
	var chestLeft  = [new THREE.Vector2(0.75,0.75),new THREE.Vector2(0.75,0.5),
					 new THREE.Vector2(0.875,0.5),new THREE.Vector2(0.875,0.75)];
	var chestFront = [new THREE.Vector2(0.5,0.75),new THREE.Vector2(0.5,0.5),
					 new THREE.Vector2(0.75,0.5),new THREE.Vector2(0.75,0.75)];
	var chestRight = [new THREE.Vector2(0.75,0.75),new THREE.Vector2(0.75,0.5),
					 new THREE.Vector2(0.875,0.5),new THREE.Vector2(0.875,0.75)];
	var chestBack  = [new THREE.Vector2(0.75,0.75),new THREE.Vector2(0.75,0.5),
					 new THREE.Vector2(1.0,0.5),new THREE.Vector2(1.0,0.75)];
	var chestTop   = [new THREE.Vector2(0.0,0.5),new THREE.Vector2(0.0,0.375),
					 new THREE.Vector2(0.25,0.375),new THREE.Vector2(0.25,0.5)];
	var chestBottom = [new THREE.Vector2(0.0,0.375),new THREE.Vector2(0.0,0.25),
					 new THREE.Vector2(0.25,0.25),new THREE.Vector2(0.25,0.375)];
					 
	//legs
	var legLeft  =  legRight = [new THREE.Vector2(0.5,0.5),new THREE.Vector2(0.5,0.25),
					 new THREE.Vector2(0.675,0.25),new THREE.Vector2(0.675,0.5)];
	var legFront = legBack = [new THREE.Vector2(0.625,0.5),new THREE.Vector2(0.625,0.25),
					 new THREE.Vector2(0.75,0.25),new THREE.Vector2(0.75,0.5)];
	var legTop   = [new THREE.Vector2(0.75,0.5),new THREE.Vector2(0.75,0.375),
					 new THREE.Vector2(0.875,0.375),new THREE.Vector2(0.875,0.5)];
	var legBottom = [new THREE.Vector2(0.75,0.375),new THREE.Vector2(0.75,0.25),
					 new THREE.Vector2(0.875,0.25),new THREE.Vector2(0.875,0.375)];
					 
	//arms
	var armLeft  =  armRight = [new THREE.Vector2(0.25,0.5),new THREE.Vector2(0.25,0.25),
					 new THREE.Vector2(0.375,0.25),new THREE.Vector2(0.375,0.5)];
	var armFront = armBack = [new THREE.Vector2(0.375,0.5),new THREE.Vector2(0.375,0.25),
					 new THREE.Vector2(0.5,0.25),new THREE.Vector2(0.5,0.5)];
	var armTop   = [new THREE.Vector2(0.875,0.5),new THREE.Vector2(0.875,0.375),
					 new THREE.Vector2(1.0,0.375),new THREE.Vector2(1.0,0.5)];
	var armBottom = [new THREE.Vector2(0.875,0.375),new THREE.Vector2(0.875,0.25),
					 new THREE.Vector2(1.0,0.25),new THREE.Vector2(1.0,0.375)];
	 
	 
	//Create the leg geometry 
	var legG = new THREE.BoxGeometry(limbW,limbH,limbD);
	//this makes the legs rotate around the right point
	legG.applyMatrix( new THREE.Matrix4().makeTranslation(0, -(limbH * 0.5), 0) );
	
	//clear the geometry's uv arrays in case there was something in it
	legG.faceVertexUvs[0] = [];
				
	//assign each face to its corresponding texture coordinates. Each face is a triangle.
	legG.faceVertexUvs[0][0] = [legRight[0],legRight[1],legRight[3]];
	legG.faceVertexUvs[0][1] = [legRight[1],legRight[2],legRight[3]];
	
	legG.faceVertexUvs[0][2] = [legLeft[0],legLeft[1],legLeft[3]];
	legG.faceVertexUvs[0][3] = [legLeft[1],legLeft[2],legLeft[3]];
	
	legG.faceVertexUvs[0][4] = [legTop[0],legTop[1],legTop[3]];
	legG.faceVertexUvs[0][5] = [legTop[1],legTop[2],legTop[3]];
	
	legG.faceVertexUvs[0][6] = [legBottom[0],legBottom[1],legBottom[3]];
	legG.faceVertexUvs[0][7] = [legBottom[1],legBottom[2],legBottom[3]];
	
	legG.faceVertexUvs[0][8] = [legFront[0],legFront[1],legFront[3]];
	legG.faceVertexUvs[0][9] = [legFront[1],legFront[2],legFront[3]];
	
	legG.faceVertexUvs[0][10] = [legBack[0],legBack[1],legBack[3]];
	legG.faceVertexUvs[0][11] = [legBack[1],legBack[2],legBack[3]];


	/**create the left leg and translate half the width and height of the leg
	so that the origin of Steve is between his feet at the bottom **/
	var legL = new THREE.Mesh(legG,steveMat);
	legL.translateX(-0.5 * limbW);
	legL.translateY(limbH);

	/**create the right leg and translate to the correct position plus a little 
	extra so that you can see where on leg begins and the other one ends **/
	var legR = new THREE.Mesh(legG,steveMat);

	legR.translateX((0.5 * limbW)+.1);
	legR.translateY(limbH);

	/*add each left leg and the right leg to the legs container */
	legs.add(legR);
	legs.add(legL);

	/*add the legs to the steve container */
	steveObject.add(legs);
	this.add(legs);  // since we now inherit


	/* Now we are going to create the torso, starting with the chest. The
	origin of the chest is in the center of the box, so that is the origin that
	we will work from to translate the arms and head to the correct position*/
	
	var chestG = new THREE.BoxGeometry(chestW,limbH,limbD);
	
	chestG.faceVertexUvs[0] = [];
	
	//assign the faces to the texture coordinates like we did for the legs			
	chestG.faceVertexUvs[0][0] = [chestRight[0],chestRight[1],chestRight[3]];
	chestG.faceVertexUvs[0][1] = [chestRight[1],chestRight[2],chestRight[3]];
	
	chestG.faceVertexUvs[0][2] = [chestLeft[0],chestLeft[1],chestLeft[3]];
	chestG.faceVertexUvs[0][3] = [chestLeft[1],chestLeft[2],chestLeft[3]];
	
	chestG.faceVertexUvs[0][4] = [chestTop[0],chestTop[1],chestTop[3]];
	chestG.faceVertexUvs[0][5] = [chestTop[1],chestTop[2],chestTop[3]];
	
	chestG.faceVertexUvs[0][6] = [chestBottom[0],chestBottom[1],chestBottom[3]];
	chestG.faceVertexUvs[0][7] = [chestBottom[1],chestBottom[2],chestBottom[3]];
	
	chestG.faceVertexUvs[0][8] = [chestFront[0],chestFront[1],chestFront[3]];
	chestG.faceVertexUvs[0][9] = [chestFront[1],chestFront[2],chestFront[3]];
	
	chestG.faceVertexUvs[0][10] = [chestBack[0],chestBack[1],chestBack[3]];
	chestG.faceVertexUvs[0][11] = [chestBack[1],chestBack[2],chestBack[3]];
	
	
	
	
	var chest = new THREE.Mesh(chestG,steveMat);
	torso.add(chest);    

	/* add the left arm to the torso and move it -7.75 to the left because we must
	move out 1/2 the width of the chest +1/2 the width of the arm plus a little 
	breathing room to see where one box begins and where the other ends. */
	
	var armG = new THREE.BoxGeometry(limbW * .75,limbH,limbD);
	armG.applyMatrix( new THREE.Matrix4().makeTranslation(0, -(limbH * 0.5), 0) );
	
	armG.faceVertexUvs[0] = [];
				
	armG.faceVertexUvs[0][0] = [armRight[0],armRight[1],armRight[3]];
	armG.faceVertexUvs[0][1] = [armRight[1],armRight[2],armRight[3]];
	
	armG.faceVertexUvs[0][2] = [armLeft[0],armLeft[1],armLeft[3]];
	armG.faceVertexUvs[0][3] = [armLeft[1],armLeft[2],armLeft[3]];
	
	armG.faceVertexUvs[0][4] = [armTop[0],armTop[1],armTop[3]];
	armG.faceVertexUvs[0][5] = [armTop[1],armTop[2],armTop[3]];
	
	armG.faceVertexUvs[0][6] = [armBottom[0],armBottom[1],armBottom[3]];
	armG.faceVertexUvs[0][7] = [armBottom[1],armBottom[2],armBottom[3]];
	
	armG.faceVertexUvs[0][8] = [armFront[0],armFront[1],armFront[3]];
	armG.faceVertexUvs[0][9] = [armFront[1],armFront[2],armFront[3]];
	
	armG.faceVertexUvs[0][10] = [armBack[0],armBack[1],armBack[3]];
	armG.faceVertexUvs[0][11] = [armBack[1],armBack[2],armBack[3]];
	
	
	
	
	var armL = new THREE.Mesh(armG,steveMat);
	armL.translateX(-(chestW * 0.5)-(0.5*(limbW * .75)));
	armL.translateY(limbH*0.5);
	torso.add(armL);


	var armR = new THREE.Mesh(armG,steveMat);
	torso.add(armR);
	armR.translateX((chestW * 0.5)+ (0.5*(limbW * .75))); 
	armR.translateY(limbH*0.5);   

	upperBody.add(torso);

 
	//head
	var headG = new THREE.BoxGeometry( 10, 10, 10);
	
	headG.faceVertexUvs[0] = [];
				
	headG.faceVertexUvs[0][0] = [headRight[0],headRight[1],headRight[3]];
	headG.faceVertexUvs[0][1] = [headRight[1],headRight[2],headRight[3]];
	
	headG.faceVertexUvs[0][2] = [headLeft[0],headLeft[1],headLeft[3]];
	headG.faceVertexUvs[0][3] = [headLeft[1],headLeft[2],headLeft[3]];
	
	headG.faceVertexUvs[0][4] = [headTop[0],headTop[1],headTop[3]];
	headG.faceVertexUvs[0][5] = [headTop[1],headTop[2],headTop[3]];
	
	headG.faceVertexUvs[0][6] = [headBottom[0],headBottom[1],headBottom[3]];
	headG.faceVertexUvs[0][7] = [headBottom[1],headBottom[2],headBottom[3]];
	
	headG.faceVertexUvs[0][8] = [headFront[0],headFront[1],headFront[3]];
	headG.faceVertexUvs[0][9] = [headFront[1],headFront[2],headFront[3]];
	
	headG.faceVertexUvs[0][10] = [headBack[0],headBack[1],headBack[3]];
	headG.faceVertexUvs[0][11] = [headBack[1],headBack[2],headBack[3]];
	
				
	var head = new THREE.Mesh(headG,steveMat);
	//change this to limbH etc instead of numbers
	head.translateY(12.6);

	upperBody.add(head);

	upperBody.translateY(22.6); 


	steveObject.add(upperBody);
	this.add(upperBody);  // since we now inherit

	
	//GLOBAL VARIABLES FOR ANIMATION
	
	var current = null;; //current key pressed
	var last = null; //last key pressed
	//var moving = false;	
	var keyPressed = null;
	
	var walkDirection = 1; //this is the direction that his arms and legs rotate
	var rotationX = 0; //this is the rotation of his arms and legs
	var walkDx = .07; //this is the amount that the rotation of his arms and legs rotate each time the scene renders
	
	
	this.moving = function(key) {
	  	
	  	last = current;
	
		switch ( key) {
			
			case(null):
				keyPressed = null;
				break;
				
			case ("w"): // up  move forward
				keyPressed = key;
				//console.log("Hey there");
				//console.log("W keyPressed... " +keyPressed);
				current = "forward";
				break;

			case ("a"): // left
				keyPressed = key;
			    current = "left";
				break;

			case ("d"): // right
				keyPressed = key;
				current = "right";
				break;

		}
		
	}
	
	
	
	
	this.rotate = function() {
		if (current == "right"){
    		// if (last == "right"){ //you do not need to rotate
//     			this.rotation.y = this.rotation.y;
//     		}
    		if (last == "forward"){
    			this.rotation.y += Math.PI / 4.0;
    		}
    		if (last == "left"){
    			this.rotation.y += Math.PI / 2.0;
    		}  
    	}
    	
    	if (current == "left"){
    		//if (last == "left") you do not need to rotate
    		if (last == "forward"){
    			this.rotation.y += - Math.PI / 4.0;
    			//console.log("Last is forward, steve.rotation.y is "+ (steve.rotation.y * (Math.PI /180.0))); 
    		}
    		if (last == "right"){
    			this.rotation.y += - Math.PI / 2.0;
    			//console.log("Last is right, steve.rotation.y is "+ (steve.rotation.y * (Math.PI /180.0))); 
    		}  
    	}
	}
	
	this.move = function(){
		var stepSize = 2;
    	var direction = this.rotation.y;
    	var dz = Math.cos(direction)* stepSize;
    	var dx = Math.sin(direction)* stepSize;
    	this.position.x += dx;
    	this.position.z += dz;
	}
	
	this.walk = function(){
		if (walkDirection == 1){
				rotationX += walkDx;
			}
			
			if (walkDirection == -1){
				rotationX -= walkDx;
			}
			
			if (rotationX >= (1) || rotationX <= (-1)){
				walkDirection = walkDirection * -1; 
			}
			
			legL.rotation.x = rotationX; //this is in radians!!!
 			legR.rotation.x = -rotationX;
 			armR.rotation.x = rotationX; //this is in radians!!!
 			armL .rotation.x = -rotationX; 	
	}
	
	this.update = function(){
		//console.log("Update: "+ keyPressed);
		if (keyPressed!= null){
			this.walk(); 
			this.rotate();
			this.move();
		}
	}

	return this;

};


Steve.prototype = Object.create( THREE.Object3D.prototype );
